<?php
/**
 * Functions file.
 * 
 * To our understanding, this can be used to add arbitrary PHP code to our theme
 * (very similar to a plugin) and to tell WordPress about the abilities of our theme.
 * 
 * @see https://developer.wordpress.org/themes/basics/theme-functions/
 */

// We support post thumbnails (now known as "featured images"). A post thumbnail is displayed
// in the header of a post.
// If there is no post thumbnail specified, we fall back to a default header image...
add_theme_support('post-thumbnails');

// ...which can also be customized (site-wide)
add_theme_support('custom-header', array(
    'default-image' => get_template_directory_uri().'/img/header.jpg', 
    'uploads'       => true)
);

// And we also support a custom menu in the header
// See https://developer.wordpress.org/reference/functions/register_nav_menu/
function register_fscoli_header_menu() {
    register_nav_menu('header-menu', 'Header-Menü');
}
add_action('init', 'register_fscoli_header_menu');

// We also allow the uploading of TeX files, which are blocked by default
function fscoli_mime_types($mime_types){
    $mime_types['tex'] = 'text/x-tex'; //TeX files
    return $mime_types;
}
add_filter('upload_mimes', 'fscoli_mime_types', 1, 1);

/* 
 * Add support to embed umap (umap.openstreetmap.de)
 * See https://wordpress.org/support/article/embeds/#adding-support-for-a-non-oembed-site
 * For more details on the parameters this function is called with, see also
 * https://developer.wordpress.org/reference/classes/wp_embed/get_embed_handler_html/
 */
function fscoli_embed_umap($regex_matches, $embed_attrs, $url, $raw_embed_attrs) {
    $width = esc_attr(
        is_numeric($embed_attrs['width']) ? $embed_attrs['width']."px" : "auto"
    );

    $height = esc_attr(
        is_numeric($embed_attrs['height']) ? $embed_attrs['height']."px" : "auto"
    );
    
    $url = esc_url($url);
    return <<<HTML
        <iframe src="$url" style="width: $width; height: $height" allow="fullscreen"></iframe>
    HTML;
}
wp_embed_register_handler('umap', '#^https://umap.openstreetmap.de/[a-z]{2}/map/[^/]*#', 'fscoli_embed_umap');

/**
 * Helper function: Matches the given regex against the given string and
 * returns the $group-th match group of the match. Returns null if there is
 * no such group or there was no match.
 */
function fscoli_regex_match_group($regex, $str, $group) {
    preg_match($regex, $str, $matches);
    /* 
     * Q: Wait, where did $matches suddenly come from?
     * A: We passed it by reference (https://www.php.net/manual/en/language.references.pass.php). This means
     * we told the function it should store its results in a variable we can later access as $matches.
     * Yes, that's a thing in some programming languages.
     */ 
    if(array_key_exists($group, $matches)) {
        return $matches[$group];
    } else {
        return null;
    }
}

/**
 * Helper function: Builds a consent prompt (HTML and JS) using the given parameters, that already have to be extracted
 * from somewhere.
 */
function fscoli_get_consent_prompt($embed_html, $width_attr, $height_attr, $content_name, $provider_url, $provider_domain, $privacy_policy_url) {
    $embed_html = esc_html($embed_html);
    $width_attr = esc_attr($width_attr);
    $height_attr = esc_attr($height_attr);
    $content_name = esc_html($content_name);
    $provider_url = esc_url($provider_url);
    $provider_domain = esc_html($provider_domain);
    
    if($privacy_policy_url !== null) {
        $privacy_policy_url = esc_url($privacy_policy_url);
        $privacy_policy_link = <<<HTML
        <a href="$privacy_policy_url">dort verlinkte Datenschutzerklärung</a>
        HTML;
    } else {
        $privacy_policy_link = "dort verlinkte Datenschutzerklärung";
    }

    /* 
     * We use the heredoc syntax (https://www.php.net/manual/en/language.types.string.php#language.types.string.syntax.heredoc)
     * to embed a multiline string in this document. Heredocs can be delimited by any string - we chose HTML, but EOT and END
     * are more commonly used options. Funny enough, when we use HTML to delimit the heredoc, at least Visual Studio Code
     * applies HTML syntax highlighting to the string...
     */ 
    return <<<HTML
    <div class="privacy-consent-prompt-area" style="width: $width_attr; height: $height_attr;">
        <div class="unloaded-content">
            $embed_html
        </div>

        <section class="privacy-consent-prompt">
            <button onclick="givePrivacyConsent(this.parentElement.parentElement)">$content_name anzeigen</button>
            <p>Dabei werden Daten an <a href="$provider_url">$provider_domain</a> (und ggfs. weitere Anbieter) übertragen. Es gilt die $privacy_policy_link.</p>
        </section>
    </div>

    <script>
        function givePrivacyConsent(consentPromptArea) {
            consentPromptArea.outerHTML = consentPromptArea.querySelector(".unloaded-content").textContent
        }
    </script>
    HTML;
}

/*
 * Add privacy protection for embed elements: You have to click a button of a consent prompt
 * before the content is loaded.
 */
function fscoli_show_embed_privacy_consent_prompt($embed_html) {
    // Try to guess width and height of the consent prompt to match the dimensions of the embedded content
    // We try to extract them from CSS properties (e.g. width: 150px) or HTML attributes (width="150px").
    $width              = fscoli_regex_match_group('/width ?(:|= ?["\']) ?(\d+ ?px)/', $embed_html, 2);
    $height             = fscoli_regex_match_group('/height ?(:|= ?["\']) ?(\d+ ?px)/', $embed_html, 2);
    if($width === null) { $width = "auto"; }
    if($height === null) { $height = "auto"; }

    // Try to guess the URL of the content provider, its domain and, using a hardcoded list of policies,
    // its privacy policy.
    $provider_url       = fscoli_regex_match_group('#src ?= ?["\'](https?://(.*?))/.*?["\']#', $embed_html, 1);
    $provider_domain    = fscoli_regex_match_group('#src ?= ?["\'](https?://(.*?))/.*?["\']#', $embed_html, 2);

    $privacy_policy_url = null;
    $content_name = "Inhalt";
    if($provider_domain === "umap.openstreetmap.de") {
        $privacy_policy_url = "https://www.fossgis.de/datenschutzerkl%C3%A4rung";
        $content_name = "Karte";
    }

    return fscoli_get_consent_prompt($embed_html, $width, $height, $content_name, $provider_url, $provider_domain, $privacy_policy_url);   
}

// To disable the privacy consent prompts (and always show the content directly), remove the line below
add_filter('embed_handler_html', 'fscoli_show_embed_privacy_consent_prompt');

/**
 * Require authentication for every REST endpoint.
 * 
 * See: https://developer.wordpress.org/rest-api/frequently-asked-questions/#require-authentication-for-all-requests
 * 
 * This prevents things like trivial user enumeration, see here:
 * https://www.gosecure.net/blog/2021/03/16/6-ways-to-enumerate-wordpress-users/
 */
function fscoli_require_authentication_for_rest_api($previous_result) {
    if($previous_result !== null || is_user_logged_in()) {
        // If the result is not null, a previous filter has already performed an
        // authentication check, and we leave the result of that filter untouched.

        // If the user is logged in, we also leave the result of the previous filter untouched.
        return $previous_result;
    } else {
        // Only when there is no previous result and the user is not logged in, we complain.
        return new WP_Error('fscoli_rest_authentication_required', __('Authentication is required to use the REST API'), ['status' => 401]);
    }
}

// To not require the authentication for all REST requests, remove the line below
add_filter('rest_authentication_errors', 'fscoli_require_authentication_for_rest_api');

/**
 * Disable the sitemap for users.
 * 
 * See all sitemaps here: https://fachschaft.cl.uni-heidelberg.de/wp-sitemap.xml
 */
function fscoli_disable_users_sitemap($provider, $provider_name) {
    if($provider_name === "users") {
        return false;
    } else {
        return $provider;
    }
}

// To re-enable /wp-sitemap-users-1.xml and /en/wp-sitemap-users-1.xml, remove the line below
add_filter('wp_sitemaps_add_provider', 'fscoli_disable_users_sitemap', accepted_args: 2);

wp_enqueue_script('fscoli-utils', get_template_directory_uri() . '/js/utils.js');
wp_enqueue_script('fscoli-pills', get_template_directory_uri() . '/js/pills.js');
?>
